//========================================================================================
//
//	File:				Command.h
//	Release Version:	$ ODF 1 $
//
//	Author:				M.Boetcher
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef COMMAND_H
#define COMMAND_H

// ----- Part Layer -----

#ifndef FWCLPCMD_H
#include "FWClpCmd.h"
#endif

#ifndef FWINSCMD_H
#include "FWInsCmd.h"
#endif

#ifndef FWDRCMD_H
#include "FWDrCmd.h"
#endif

//========================================================================================
// Forward Declarations
//========================================================================================

class CEmbedContent;
class CEmbedFrame;
class CEmbedSelection;
class CEmbedProxy;

//========================================================================================
// CEmbedEditCommand
//========================================================================================

class CEmbedEditCommand : public FW_CClipboardCommand
{
  public:

	FW_DECLARE_AUTO(CEmbedEditCommand)
	
	CEmbedEditCommand(Environment* ev, 
					  ODCommandID commandID,
					  CEmbedContent* content, 
					  FW_CFrame* frame, 
					  FW_CSelection* selection,
					  FW_Boolean canUndo);

	virtual ~CEmbedEditCommand();

	// --- FW_CCommand overrides
	virtual void UndoIt(Environment* ev);			// Override
	virtual void RedoIt(Environment* ev);			// Override
	virtual void SaveUndoState(Environment* ev);	// Override
	virtual void SaveRedoState(Environment* ev);	// Override
	virtual void FreeUndoState(Environment* ev);	// Override
	virtual void FreeRedoState(Environment* ev);	// Override

  private:
	void RemovePart(Environment* ev);
	void RestorePart(Environment* ev);
	void RestoreOldPart(Environment* ev);

  private:
	CEmbedContent*		fEmbedContent;
	CEmbedProxy*		fPastedProxy;	// saved proxy for Redo
	CEmbedProxy*		fOldProxy;		// saved proxy for Undo
};

//========================================================================================
//	class CEmbedInsertCommand
//========================================================================================

class CEmbedInsertCommand : public FW_CInsertCommand
{
  public:
  
	FW_DECLARE_AUTO(CEmbedInsertCommand)

	CEmbedInsertCommand(Environment* ev,
						FW_CEmbeddingFrame* frame,
						const FW_PFileSpecification& fileSpec,
						CEmbedContent* content);

	virtual ~CEmbedInsertCommand();

	// --- FW_CCommand overrides
	virtual void		UndoIt(Environment* ev);	// Override
	virtual void		RedoIt(Environment* ev);	// Override
	virtual void		CommitDone(Environment* ev);	// Override
	virtual void		CommitUndone(Environment* ev);	// Override
	virtual void		SaveUndoState(Environment* ev);	// Override
	virtual void		SaveRedoState(Environment* ev);	// Override

  private:
	CEmbedContent*		fPartContent;
	CEmbedSelection*	fEmbedSelection;
	CEmbedProxy*		fInsertedProxy;	// saved proxy for Redo
	CEmbedProxy*		fOldProxy;		// saved proxy for Undo
};

//========================================================================================
//	class CEmbedDropCommand
//========================================================================================

class CEmbedDropCommand : public FW_CDropCommand
{
  public:
  
	FW_DECLARE_AUTO(CEmbedDropCommand)
	
	CEmbedDropCommand(Environment* ev,
					  CEmbedContent* content,
					  FW_CFrame* frame,
					  ODDragItemIterator* dropInfo, 
					  ODFacet* odFacet,
					  const FW_CPoint& dropPoint);

	virtual ~ CEmbedDropCommand();

	// --- for Undo/Redo ---
	virtual void		UndoIt(Environment* ev);	// Override
	virtual void		RedoIt(Environment* ev);	// Override
	virtual void		CommitDone(Environment* ev);	// Override
	virtual void		CommitUndone(Environment* ev);	// Override
	virtual void		SaveUndoState(Environment* ev);	// Override
	virtual void		SaveRedoState(Environment* ev);	// Override

  private:
	CEmbedContent*		fEmbedContent;
	CEmbedSelection*	fEmbedSelection;
	CEmbedProxy*		fDroppedProxy;	// saved proxy for Redo
	CEmbedProxy*		fOldProxy;		// saved proxy for Undo
};

//========================================================================================
//	class CEmbedDragCommand
//========================================================================================

class CEmbedDragCommand : public FW_CDragCommand
{
  public:
  
  	FW_DECLARE_AUTO(CEmbedDragCommand)
	
	CEmbedDragCommand(Environment* ev,
					CEmbedContent* content,
					FW_CFrame* frame,
					CEmbedSelection* selection);

	virtual ~ CEmbedDragCommand();

	// --- FW_CCommand overrides for Undo/Redo ---
	virtual void		UndoIt(Environment* ev);			// Override
	virtual void		RedoIt(Environment* ev);			// Override
	virtual void		SaveUndoState(Environment* ev);		// Override
	virtual void		FreeUndoState(Environment* ev);		// Override

  private:
	CEmbedContent*		fEmbedContent;
	CEmbedSelection*	fEmbedSelection;
	CEmbedProxy*		fSavedProxy;
	CEmbedProxy*		fDraggedProxy;
};


#endif
