//========================================================================================
//
//	File:				Frame.h
//	Release Version:	$ ODF 1 $
//
//	Author:				M.Boetcher
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FRAME_H
#define FRAME_H

// ----- Framework Includes -----

#ifndef FWFRMING_H
#include "FWFrming.h"
#endif

#ifndef FWDRGDRP_H
#include "FWDrgDrp.h"
#endif

//========================================================================================
// Forward class declaration
//========================================================================================

class CEmbedPart;
class CEmbedContent;

//========================================================================================
// CLASS CEmbedFrame
//========================================================================================

class CEmbedFrame : public FW_CEmbeddingFrame, public FW_MDraggableFrame, public FW_MDroppableFrame
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
  public:
	FW_DECLARE_AUTO(CEmbedFrame)
	
	CEmbedFrame(Environment* ev, ODFrame* odFrame, FW_CPresentation* presentation, 
				CEmbedPart* part, CEmbedContent* content);
	virtual ~CEmbedFrame();

//----------------------------------------------------------------------------------------
//	Inherited API
//
  public:
	virtual void 					Draw(Environment* ev, ODFacet* odFacet, ODShape* invalidShape);

	virtual void 					CreateSubViews(Environment* ev);

	virtual FW_Boolean 				DoAdjustMenus(Environment* ev, 
											FW_CMenuBar* menuBar, 
											FW_Boolean hasMenuFocus, 
											FW_Boolean isRoot);

	virtual ODFacet*				CreateEmbeddedFacet(Environment* ev,
											ODFacet* embeddingFacet,
											FW_MProxy* proxy,
											ODFrame* embeddedFrame,
											ODShape* proposedClipShape);
										
	virtual void					FrameShapeChanged(Environment* ev);

	virtual ODDragResult			CanAcceptDrop(Environment* ev, ODDragItemIterator* dragInfo);

	virtual FW_CClipboardCommand*	NewClipboardCommand(Environment* ev, 
											ODCommandID commandID);

	virtual FW_CDragCommand*		NewDragCommand(Environment* ev, 
											FW_CFrame* theFrame, 
											const FW_CMouseEvent& theMouseEvent);
											
	virtual FW_CDropCommand* 		NewDropCommand(Environment* ev,
											FW_CFrame* frame,
											ODDragItemIterator* dropInfo, 
											ODFacet* odFacet, 
											const FW_CPoint& dropPoint);

	virtual FW_CInsertCommand*		NewInsertCommand(Environment* ev, const FW_PFileSpecification& fileSpec);

	virtual FW_Boolean				DoMouseDownInEmbeddedFrameBorder(Environment* ev, 
											const FW_CBorderMouseEvent& theBorderMouseEvent);
											
//----------------------------------------------------------------------------------------
//	New API
//
  public:
	ODShape*						MakeFrameShape(Environment* ev);

	ODFacet*						CreateOneEmbeddedFacet(Environment* ev, 
											ODFacet* embeddingFacet, 
											ODFrame* embeddedFrame, 
											ODFacet* sibling,
											const FW_CRect& clipRect);

	void							EmbeddedFacetBounds(Environment* ev, FW_CRect* clipRects);
										
//----------------------------------------------------------------------------------------
//	Data Members
//
  private:
	CEmbedPart*			fPart;
	CEmbedContent* 		fContent;
};

#endif
